from threading import Thread
from . import worker
from multiprocessing import cpu_count
import time
import sys
import os

_THREADS_MAX = cpu_count()


def threaded_worker(routing_graph, routing_queue, user_to, progress):
    while True:
        try:
            found, queued = worker.explore_worker(routing_queue.pop(0))
        except IndexError:
            continue
        for new_node in found:
            if new_node not in routing_graph.keys():
                routing_graph[new_node] = queued
            routing_queue.append(new_node)
        progress += len(found)
        if user_to in routing_graph.keys():
            break


def route(user_from: str, user_to: str) -> None:
    """Main function.

    Works as a direct executor, returns nothing."""

    print('Input validation...')
    if user_to == user_from:
        print(f'Traced route: {user_from} ↔ {user_to}')
    if not (worker.check_worker(user_from) or worker.check_worker(user_to)):
        print('ROUTING FAILED! Invalid one or both usernames.')

    # initial data
    routing_graph = {}
    routing_queue = [user_from]
    progress_counter = 0

    # create threaded workers
    threads = []
    for i in range(_THREADS_MAX - 1):
        threads.append(Thread(target=threaded_worker, args=(
            routing_graph, routing_queue, user_to, progress_counter)))
        threads[-1].start()

    # wait stable connections
    init_time = time.time_ns()
    print('Workers preparation...')
    while not len(routing_queue) > 1:
        pass

    # progress output
    os.system('cls')
    while not user_to in routing_graph.keys():
        time.sleep(0.2)
        uptime = round((time.time_ns() - init_time) / 1e9, 1)
        sys.stdout.write('\033[F\033[K')
        sys.stdout.flush()
        print(f'{len(routing_queue)} positions in queue...\nUptime {uptime} seconds...', end='')
    print('')

    # make list with a route based on routing_graph
    def recursively_disclose_route(sequent, rt=None) -> list:
        if rt is None:
            rt = []
        if not sequent == user_from:
            recursively_disclose_route(routing_graph[sequent], rt).append(sequent)
        else:
            rt.append(user_from)
        return rt

    print('Traced route:', ' ↔ '.join(recursively_disclose_route(user_to)))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['scrouter']

package_data = \
{'': ['*']}

install_requires = \
['requests']

setup_kwargs = {
    'name': 'ky.scrouter',
    'version': '1.0',
    'description': 'A tool that looks for relations between users',
    'long_description': '# Scrouter\n\nA tool that looks for relations between users.\n\n## Installing\n\nVia pip:\n\n```\npip install ky.scrouter\n```\n\nYou can also update to the latest version:\n\n```\npip install ky.scrouter --upgrade\n```\n\n**Make sure you have Python 3.8 or above installed!**\n\n## Program launch\n\nVia terminal:\n\n```\npython -m ky.scrouter\n```\n\n## License\nThis project follows MIT license (see [LICENSE](LICENSE)).\n',
    'author': 'Leon "Procybit" Shepelev',
    'author_email': 'kyleusnaff@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import requests
import json


def check_user(username):
    res = {'code': 'NotFound', 'message': ''}
    for i in range(10):
        try:
            res = json.loads(requests.get(
                f'https://api.scratch.mit.edu/users/{username}'
            ).text)
        except requests.exceptions.ConnectionError:
            continue

    try:
        if res['code'] == 'NotFound':
            return False
    except KeyError:
        return True


def get_followers(username):
    followers = []
    offset = 0
    while True:
        check = len(followers)

        try:
            followers += list(part['username'] for part in json.loads(requests.get(
                f'https://api.scratch.mit.edu/users/{username}/followers?limit=40&offset={offset}'
            ).text))
        except requests.exceptions.ConnectionError:
            continue

        offset += 40

        if check == len(followers) or check > 500:
            break
    return followers


def get_following(username):
    following = []
    offset = 0
    while True:
        check = len(following)

        following += list(part['username'] for part in json.loads(requests.get(
            f'https://api.scratch.mit.edu/users/{username}/following?limit=40&offset={offset}'
        ).text))
        offset += 40

        if check == len(following) or check > 500:
            break
    return following



from . import scratch_api as api


def explore_worker(username) -> tuple[tuple, str]:
    """Scratch Router's worker.

    Searches for related users (followers and following) by username using Scratch API."""

    followers = api.get_followers(username)
    following = api.get_following(username)
    friends = []

    if following > followers:
        for f in followers:
            if f in following:
                friends.append(f)
    else:
        for f in following:
            if f in followers:
                friends.append(f)

    found = following + followers

    return found, username


def check_worker(username) -> bool:
    """Scratch Router's worker.

    Checks if certain user really exists."""

    return api.check_user(username)
